'use strict'

Object.defineProperty(exports, '__esModule', {
  value: true
})
exports.default = void 0

var _fs = _interopRequireDefault(require('fs'))

var _path = require('path')

function _interopRequireDefault(obj) {
  return obj && obj.__esModule ? obj : { default: obj }
}

const fa = _fs.default.promises

function normalizeOptions({
  resolve = false,
  isExcludedDir = () => false
} = {}) {
  return {
    resolve,
    isExcludedDir
  }
}

function normalizeDirname(dirname, options) {
  if (options.resolve === true) {
    dirname = (0, _path.resolve)(dirname)
  }

  if (dirname.length > 0 && dirname[dirname.length - 1] !== _path.sep) {
    dirname += _path.sep
  }

  return dirname
}

function* sync(filename, options) {
  options = normalizeOptions(options)

  if (!_fs.default.lstatSync(filename).isDirectory()) {
    return filename
  }

  yield* (function* walk(dirname) {
    if (options.isExcludedDir(dirname)) {
      return
    }

    for (const dirent of _fs.default.readdirSync(dirname, {
      withFileTypes: true
    })) {
      const filename = dirname + dirent.name

      if (dirent.isDirectory()) {
        yield* walk(filename + _path.sep)
      } else {
        yield filename
      }
    }
  })(normalizeDirname(filename, options))
}

sync.array = (filename, options) => Array.from(sync(filename, options))

function createNotifier() {
  let done = false

  let resolve = () => {}

  let reject = () => {}

  let notified = new Promise((pResolve, pReject) => {
    resolve = pResolve
    reject = pReject
  })
  return {
    resolve() {
      const oldResolve = resolve
      notified = new Promise((pResolve, pReject) => {
        resolve = pResolve
        reject = pReject
      })
      oldResolve()
    },

    reject(error) {
      reject(error)
    },

    get done() {
      return done
    },

    set done(value) {
      done = value
    },

    onResolved() {
      return notified
    }
  }
}

function walk(dirnames, filenames, notifier, options) {
  if (dirnames.length === 0) {
    notifier.done = true
    return
  }

  const children = []
  let pending = dirnames.length

  for (const dirname of dirnames) {
    if (options.isExcludedDir(dirname)) {
      continue
    }

    _fs.default.readdir(
      dirname,
      {
        withFileTypes: true
      },
      (error, dirents) => {
        if (error != null) {
          notifier.reject(error)
          return
        }

        for (const dirent of dirents) {
          const filename = dirname + dirent.name

          if (dirent.isDirectory()) {
            children.push(filename + _path.sep)
          } else {
            filenames.push(filename)
          }
        }

        notifier.resolve()

        if (--pending === 0) {
          walk(children, filenames, notifier)
        }
      }
    )
  }
}

async function* async(filename, options) {
  options = normalizeOptions(options)

  if (!(await fa.lstat(filename)).isDirectory()) {
    return filename
  }

  const filenames = []
  const notifier = createNotifier()
  walk([normalizeDirname(filename, options)], filenames, notifier, options)

  do {
    await notifier.onResolved()

    while (filenames.length > 0) {
      yield filenames.pop()
    }
  } while (!notifier.done)
}

async.array = async (filename, options) => {
  const iterator = async(filename, options)
  const filenames = []

  for await (const filename of iterator) {
    filenames.push(filename)
  }

  return filenames
}

var _default = {
  async,
  sync
}
exports.default = _default
